from __future__ import annotations

import logging
from typing import Literal

import requests

logger = logging.getLogger(__name__)


class Jusogokr:
    """도로명 주소 API"""

    def __init__(self, api_key: str) -> None:
        self.api_key: str = api_key

    @staticmethod
    def raise_for_status(status: dict):
        code = status.get("errorCode", "0")
        if code != "0":
            raise ValueError(f'[{code}] {status.get("errorMessage", "")}')

    def addr(
        self,
        keyword: str,  # 주소 검색어
        currentPage: int = 1,  # 요청 페이지 번호
        countPerPage: int = 10,  # 페이지당 출력 할 결과 Row 수
        firstSort: Literal["none", "road", "location"] = "none",  # 정확도 정렬 (none), 우선정렬(road: 도로명 포함, location: 지번 포함)
        hstryYn: Literal["Y", "N"] = "N",  # 변동된 주소정보 포함 여부
        addInfoYn: Literal["Y", "N"] = "N",  # 출력결과에 추가된 항목(hstryYn, relJibun, hemdNm) 제공여부
    ) -> dict:
        """검색API: 도로명주소"""
        if keyword := keyword.strip():
            # https://business.juso.go.kr/addrlink/openApi/searchApi.do
            url = "https://business.juso.go.kr/addrlink/addrLinkApi.do"
            params = dict(
                confmKey=self.api_key,
                keyword=keyword,
                currentPage=f"{currentPage}",
                countPerPage=f"{countPerPage}",
                firstSort=firstSort,
                hstryYn=hstryYn,
                addInfoYn=addInfoYn,
                resultType="json",
            )
            resp = requests.get(url, params=params)
            parsed = resp.json()
            results = parsed.get("results", {})
            self.raise_for_status(results.get("common", {}))
            return results
        return {}

    def addr_coord(
        self,
        admCd: str,  # 행정구역코드
        rnMgtSn: str,  # 도로명코드
        udrtYn: Literal["0", "1"],  # 지하여부(0:지상, 1:지하)
        buldMnnm: str,  # 건물본번
        buldSlno: str,  # 건물부번
        **kwargs,
    ) -> dict:
        """검색API: 좌표제공"""
        # https://business.juso.go.kr/addrlink/openApi/searchApi.do
        url = "https://business.juso.go.kr/addrlink/addrCoordApi.do"
        params = dict(
            confmKey=self.api_key,
            admCd=admCd,
            rnMgtSn=rnMgtSn,
            udrtYn=udrtYn,
            buldMnnm=buldMnnm,
            buldSlno=buldSlno,
            resultType="json",
        )
        resp = requests.get(url, params=params)
        parsed = resp.json()
        results = parsed.get("results", {})
        self.raise_for_status(results.get("common", {}))
        return results



from __future__ import annotations

import logging
from typing import Literal

import requests

logger = logging.getLogger(__name__)


def _query_category(cid: int = 0) -> dict:
    url = "https://datalab.naver.com/shoppingInsight/getCategory.naver"
    headers = {
        "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/125.0.0.0 Safari/537.36",
        "Referer": "https://datalab.naver.com/shoppingInsight/sKeyword.naver",
    }
    resp = requests.get(url, params=dict(cid=cid), headers=headers)
    resp.raise_for_status()
    return resp.json()


class Naver:
    """NAVER Service API"""

    def __init__(self, api_key: str, api_sec: str) -> None:
        self.api_key: str = api_key
        self.api_sec: str = api_sec

    @property
    def headers(self) -> dict:
        return {"X-Naver-Client-Id": self.api_key, "X-Naver-Client-Secret": self.api_sec}

    def adult(self, query: str) -> dict:
        """성인 검색어 판별 결과 조회"""
        if query := query.strip():
            # https://developers.naver.com/docs/serviceapi/search/adult/adult.md
            url = "https://openapi.naver.com/v1/search/adult.json"
            params = {"query": query}
            resp = requests.get(url, params=params, headers=self.headers)
            resp.raise_for_status()
            return resp.json()
        return {}

    def blog(self, query: str, display: int = 10, start: int = 1, sort: Literal["sim", "date"] = "sim") -> dict:
        """블로그 검색 결과 조회"""
        if query := query.strip():
            # https://developers.naver.com/docs/serviceapi/search/blog/blog.md
            url = "https://openapi.naver.com/v1/search/blog.json"
            params = {"query": query, "display": f"{display}", "start": f"{start}", "sort": f"{sort}"}
            resp = requests.get(url, params=params, headers=self.headers)
            resp.raise_for_status()
            return resp.json()
        return {}

    def local(self, query: str, display: int = 5, start: int = 1, sort: Literal["random", "comment"] = "random") -> dict:
        """지역 검색 결과 조회"""
        if query := query.strip():
            # https://developers.naver.com/docs/serviceapi/search/local/local.md
            url = "https://openapi.naver.com/v1/search/local.json"
            params = {"query": query, "display": f"{display}", "start": f"{start}", "sort": f"{sort}"}
            resp = requests.get(url, params=params, headers=self.headers)
            resp.raise_for_status()
            return resp.json()
        return {}

    def shop(
        self,
        query: str,
        display: int = 10,
        start: int = 1,
        sort: Literal["sim", "date", "asc", "dsc"] = "sim",
        filter: Literal["naverpay"] = None,
        exclude: Literal["used", "rental", "cbshop"] = None,
    ) -> dict:
        """쇼핑 검색 결과 조회"""
        if query := query.strip():
            # https://developers.naver.com/docs/serviceapi/search/shopping/shopping.md
            url = "https://openapi.naver.com/v1/search/shop.json"
            params = {
                "query": query,
                "display": f"{display}",
                "start": f"{start}",
                "sort": f"{sort}",
                **(dict(filter=filter) if filter else {}),
                **(dict(exclude=exclude) if exclude else {}),
            }
            resp = requests.get(url, params=params, headers=self.headers)
            resp.raise_for_status()
            return resp.json()
        return {}

    def lab_search(
        self,
        start_date: str,
        end_date: str,
        time_unit: Literal["date", "week", "month"],
        keyword_groups: list[tuple[str, tuple[str, ...]]],
        device: Literal["pc", "mo"] = None,
        gender: Literal["m", "f"] = None,
        ages: list[Literal["1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11"]] = None,
    ) -> dict:
        """네이버 통합 검색어 트렌드 조회"""
        # https://developers.naver.com/docs/serviceapi/datalab/search/search.md
        url = "https://openapi.naver.com/v1/datalab/search"
        params = {
            "startDate": start_date,
            "endDate": end_date,
            "timeUnit": time_unit,
            "keywordGroups": [{"groupName": name, "keywords": kw} for name, kw in keyword_groups],
            **(dict(device=device) if device else {}),
            **(dict(gender=gender) if gender else {}),
            **(dict(ages=ages) if ages else {}),
        }
        resp = requests.post(url, json=params, headers=self.headers)
        resp.raise_for_status()
        return resp.json()

    def lab_shopping(
        self,
        start_date: str,
        end_date: str,
        time_unit: Literal["date", "week", "month"],
        category: list[tuple[str, tuple[str, ...]]],
        device: Literal["pc", "mo"] = None,
        gender: Literal["m", "f"] = None,
        ages: list[Literal["10", "20", "30", "40", "50", "60"]] = None,
    ) -> dict:
        """쇼핑인사이트 분야별 트렌드 조회"""
        # https://developers.naver.com/docs/serviceapi/datalab/shopping/shopping.md
        url = "https://openapi.naver.com/v1/datalab/shopping/categories"
        params = {
            "startDate": start_date,
            "endDate": end_date,
            "timeUnit": time_unit,
            "category": [{"name": name, "param": kw} for name, kw in category],
            **(dict(device=device) if device else {}),
            **(dict(gender=gender) if gender else {}),
            **(dict(ages=ages) if ages else {}),
        }
        resp = requests.post(url, json=params, headers=self.headers)
        resp.raise_for_status()
        return resp.json()

    @staticmethod
    def lab_category(cid: int = 0) -> list[dict]:
        result = _query_category(cid).get("childList", [])
        return [{k: item[k] for k in ["cid", "pid", "name", "leaf"]} for item in result]



from __future__ import annotations

import logging
from calendar import monthrange
from datetime import datetime as dt
from enum import Enum

import requests

logger = logging.getLogger(__name__)


class RespType(str, Enum):
    JSON = "json"
    XML = "xml"

    def __str__(self) -> str:
        return self.value


class RespLang(str, Enum):
    KR = "kr"
    EN = "en"

    def __str__(self) -> str:
        return self.value


class RespIntv(str, Enum):
    ANNUAL = "A"
    SEMMI_ANNUAL = "S"
    QUARTERLY = "Q"
    MONTHLY = "M"
    SEMMI_MONTHLY = "SM"
    DAILY = "D"

    def __str__(self) -> str:
        return self.value


class ServName(str, Enum):
    STAT_TABLE_LIST = "StatisticTableList"
    STAT_WORD = "StatisticWord"
    STAT_ITEM_LIST = "StatisticItemList"
    STAT_SEARCH = "StatisticSearch"
    KEY_STAT_LIST = "KeyStatisticList"
    STAT_META = "StatisticMeta"

    def __str__(self) -> str:
        return self.value


def dt_to_str(now: dt, intv: str) -> str:
    if intv == RespIntv.ANNUAL:
        return now.strftime("%Y")

    if intv == RespIntv.SEMMI_ANNUAL:
        s = "S1" if now.month <= 6 else "S2"
        return now.strftime(f"%Y{s}")

    if intv == RespIntv.QUARTERLY:
        return now.strftime(f"%YQ{now.month//4 +1}")

    if intv == RespIntv.MONTHLY:
        return now.strftime("%Y%m")

    if intv == RespIntv.SEMMI_MONTHLY:
        _, days = monthrange(now.year, now.month)
        sm = "S1" if now.day <= int(days / 2) else "S2"
        return now.strftime(f"%Y%m{sm}")

    if intv == RespIntv.DAILY:
        return now.strftime("%Y%m%d")

    raise ValueError(f"invalid interval, got {intv!r}")


def str_to_dt(date_string: str, intv: str) -> dt:
    if intv == RespIntv.ANNUAL:
        return dt.strptime(date_string, "%Y")

    if intv == RespIntv.SEMMI_ANNUAL:
        year, cnt = [int(x) for x in date_string.split("S")]
        if cnt == 1:
            return dt(year=year, month=6, day=30)
        elif cnt == 2:
            return dt(year=year, month=12, day=31)
        raise ValueError(f"invalid date_string, got {date_string!r}")

    if intv == RespIntv.QUARTERLY:
        year, cnt = [int(x) for x in date_string.split("Q")]
        if cnt == 1:
            return dt(year=year, month=3, day=31)
        elif cnt == 2:
            return dt(year=year, month=6, day=30)
        elif cnt == 3:
            return dt(year=year, month=9, day=30)
        elif cnt == 4:
            return dt(year=year, month=12, day=31)
        raise ValueError(f"invalid date_string, got {date_string!r}")

    if intv == RespIntv.MONTHLY:
        return dt.strptime(date_string, "%Y%m")

    if intv == RespIntv.SEMMI_MONTHLY:
        yyyymm, cnt = date_string.split("S")
        cnt = int(cnt)
        now = dt.strptime(yyyymm, "%Y%m")
        _, days = monthrange(now.year, now.month)
        if cnt == 1:
            return now.replace(day=int(days / 2))
        elif cnt == 2:
            return now.replace(day=days)
        raise ValueError(f"invalid date_string, got {date_string!r}")

    if intv == RespIntv.DAILY:
        return dt.strptime(date_string, "%Y%m%d")

    raise ValueError(f"invalid interval, got {intv!r}")


class Ecos:
    """ECOS Open API"""

    def __init__(self, api_key: str = None, api_url: str = None) -> None:
        self.api_key = api_key if api_key else "sample"
        self.api_url = api_url if api_url else "http://ecos.bok.or.kr/api/"

    def _api_call(self, args: dict) -> dict | bytes:
        req_url = f"{self.api_url}{'/'.join(args.values())}"
        resp = requests.get(req_url)
        if args.get("요청유형", "") == RespType.JSON:
            parsed = resp.json()
            result = parsed.get("RESULT", {})
            if result:
                logger.error(args)
                raise ValueError(f"({result.get('CODE')}) {result.get('MESSAGE')}")
            return parsed
        return resp.content

    def stat_table_list(
        self,
        stat_code: str = "",
        row_from: int = 1,
        row_to: int = 100_000,
        resp_type: str = RespType.JSON,
        resp_lang: str = RespLang.KR,
        raw: bool = False,
    ) -> dict | bytes:
        """서비스 통계 목록 조회

        Args:
            stat_code (str, optional): 통계표코드. Defaults to "".
            row_from (int, optional): 요청시작건수. Defaults to 1.
            row_to (int, optional): 요청종료견수. Defaults to 100_000.
            resp_type (str, optional): 요청유형. Defaults to RespType.JSON.
            resp_lang (str, optional): 언어구분. Defaults to RespLang.KR.
            raw (bool, optional): Raw데이터여부. Defaults to False.

        Returns:
            dict | bytes: 딕셔너리(JSON 요청 시), 또는 바이트(XML 요청 시)
        """
        sname = ServName.STAT_TABLE_LIST
        args = {
            "서비스명": f"{sname}",
            "인증키": self.api_key,
            "요청유형": f"{resp_type}",
            "언어구분": f"{resp_lang}",
            "요청시작건수": f"{row_from}",
            "요청종료건수": f"{row_to}",
            "통계표코드": f"{stat_code}",
        }
        result = self._api_call(args)
        return result.get(f"{sname}", {}).get("row", []) if not raw and resp_type == RespType.JSON else result

    def stat_word(
        self,
        stat_word: str,
        row_from: int = 1,
        row_to: int = 100_000,
        resp_type: str = RespType.JSON,
        resp_lang: str = RespLang.KR,
        raw: bool = False,
    ) -> dict | bytes:
        """통계용어사전

        Args:
            stat_word (str): 용어
            row_from (int, optional): 요청시작건수. Defaults to 1.
            row_to (int, optional): 요청종료견수. Defaults to 100_000.
            resp_type (str, optional): 요청유형. Defaults to RespType.JSON.
            resp_lang (str, optional): 언어구분. Defaults to RespLang.KR.
            raw (bool, optional): Raw데이터여부. Defaults to False.

        Returns:
            dict | bytes: 딕셔너리(JSON 요청 시), 또는 바이트(XML 요청 시)
        """
        sname = ServName.STAT_WORD
        args = {
            "서비스명": f"{sname}",
            "인증키": self.api_key,
            "요청유형": f"{resp_type}",
            "언어구분": f"{resp_lang}",
            "요청시작건수": f"{row_from}",
            "요청종료건수": f"{row_to}",
            "용어": f"{stat_word}",
        }
        result = self._api_call(args)
        return result.get(f"{sname}", {}).get("row", []) if not raw and resp_type == RespType.JSON else result

    def stat_item_list(
        self,
        stat_code: str,
        row_from: int = 1,
        row_to: int = 100_000,
        resp_type: str = RespType.JSON,
        resp_lang: str = RespLang.KR,
        raw: bool = False,
    ) -> dict | bytes:
        """통계 세부항목 목록

        Args:
            stat_code (str, optional): 통계표코드. Defaults to "".
            row_from (int, optional): 요청시작건수. Defaults to 1.
            row_to (int, optional): 요청종료견수. Defaults to 100_000.
            resp_type (str, optional): 요청유형. Defaults to RespType.JSON.
            resp_lang (str, optional): 언어구분. Defaults to RespLang.KR.
            raw (bool, optional): Raw데이터여부. Defaults to False.

        Returns:
            dict | bytes: 딕셔너리(JSON 요청 시), 또는 바이트(XML 요청 시)
        """
        sname = ServName.STAT_ITEM_LIST
        args = {
            "서비스명": f"{sname}",
            "인증키": self.api_key,
            "요청유형": f"{resp_type}",
            "언어구분": f"{resp_lang}",
            "요청시작건수": f"{row_from}",
            "요청종료건수": f"{row_to}",
            "통계표코드": stat_code,
        }
        result = self._api_call(args)
        return result.get(f"{sname}", {}).get("row", []) if not raw and resp_type == RespType.JSON else result

    def stat_search(
        self,
        stat_code: str,
        intv: str = RespIntv.DAILY,
        search_from: str = "",
        search_to: str = "",
        item_code1: str = "?",
        item_code2: str = "?",
        item_code3: str = "?",
        item_code4: str = "?",
        row_from: int = 1,
        row_to: int = 100_000,
        resp_type: str = RespType.JSON,
        resp_lang: str = RespLang.KR,
        raw: bool = False,
    ) -> list | dict | bytes:
        """통계 조회 조건 설정

        Args:
            stat_code (str): 통계표코드
            intv (str, optional): 주기. Defaults to RespIntv.DAILY.
            search_from (str, optional): 검색시작일자. Defaults to "".
            search_to (str, optional): 검색종료일자. Defaults to "".
            item_code1 (str, optional): 통계항목코드1. Defaults to "?".
            item_code2 (str, optional): 통계항목코드2. Defaults to "?".
            item_code3 (str, optional): 통계항목코드3. Defaults to "?".
            item_code4 (str, optional): 통계항목코드4. Defaults to "?".
            row_from (int, optional): 요청시작건수. Defaults to 1.
            row_to (int, optional): 요청종료견수. Defaults to 100_000.
            resp_type (str, optional): 요청유형. Defaults to RespType.JSON.
            resp_lang (str, optional): 언어구분. Defaults to RespLang.KR.
            raw (bool, optional): Raw데이터여부. Defaults to False.

        Returns:
            dict | bytes: 딕셔너리(JSON 요청 시), 또는 바이트(XML 요청 시)
        """

        if not search_to:
            search_to = dt_to_str(dt.now(), intv)
        if not search_from:
            search_from = dt_to_str(dt.fromtimestamp(0), intv)
        sname = ServName.STAT_SEARCH
        args = {
            "서비스명": f"{sname}",
            "인증키": f"{self.api_key}",
            "요청유형": f"{resp_type}",
            "언어구분": f"{resp_lang}",
            "요청시작건수": f"{row_from}",
            "요청종료건수": f"{row_to}",
            "통계표코드": f"{stat_code}",
            "주기": f"{intv}",
            "검색시작일자": f"{search_from}",
            "검색종료일자": f"{search_to}",
            "통계항목코드1": f"{item_code1}",
            "통계항목코드2": f"{item_code2}",
            "통계항목코드3": f"{item_code3}",
            "통계항목코드4": f"{item_code4}",
        }
        result = self._api_call(args)
        return result.get(f"{sname}", {}).get("row", []) if not raw and resp_type == RespType.JSON else result

    def key_stat_list(
        self,
        row_from: int = 1,
        row_to: int = 100_000,
        resp_type: str = RespType.JSON,
        resp_lang: str = RespLang.KR,
        raw: bool = False,
    ) -> list | dict | bytes:
        """100대 통계지표

        Args:
            row_from (int, optional): 요청시작건수. Defaults to 1.
            row_to (int, optional): 요청종료견수. Defaults to 100_000.
            resp_type (str, optional): 요청유형. Defaults to RespType.JSON.
            resp_lang (str, optional): 언어구분. Defaults to RespLang.KR.
            raw (bool, optional): Raw데이터여부. Defaults to False.

        Returns:
            dict | bytes: 딕셔너리(JSON 요청 시), 또는 바이트(XML 요청 시)
        """
        sname = ServName.KEY_STAT_LIST
        args = {
            "서비스명": f"{ServName.KEY_STAT_LIST}",
            "인증키": f"{self.api_key}",
            "요청유형": f"{resp_type}",
            "언어구분": f"{resp_lang}",
            "요청시작건수": f"{row_from}",
            "요청종료건수": f"{row_to}",
        }
        result = self._api_call(args)
        return result.get(f"{sname}", {}).get("row", []) if not raw and resp_type == RespType.JSON else result

    def stat_meta(
        self,
        item_name: str,
        row_from: int = 1,
        row_to: int = 100_000,
        resp_type: str = RespType.JSON,
        resp_lang: str = RespLang.KR,
        raw: bool = False,
    ) -> list | dict | bytes:
        """통계메타DB

        Args:
            item_name (str): 데이터명
            row_from (int, optional): 요청시작건수. Defaults to 1.
            row_to (int, optional): 요청종료견수. Defaults to 100_000.
            resp_type (str, optional): 요청유형. Defaults to RespType.JSON.
            resp_lang (str, optional): 언어구분. Defaults to RespLang.KR.
            raw (bool, optional): Raw데이터여부. Defaults to False.

        Returns:
            dict | bytes: 딕셔너리(JSON 요청 시), 또는 바이트(XML 요청 시)
        """
        sname = ServName.STAT_META
        args = {
            "서비스명": f"{sname}",
            "인증키": f"{self.api_key}",
            "요청유형": f"{resp_type}",
            "언어구분": f"{resp_lang}",
            "요청시작건수": f"{row_from}",
            "요청종료건수": f"{row_to}",
            "데이터명": f"{item_name}",
        }
        result = self._api_call(args)
        return result.get(f"{sname}", {}).get("row", []) if not raw and resp_type == RespType.JSON else result



from __future__ import annotations

import logging

import requests

logger = logging.getLogger(__name__)


class Kakao:
    """Kakao API"""

    def __init__(self, api_key: str) -> None:
        self.api_key: str = api_key

    def local_keyword(
        self,
        query: str,
        format: str = "JSON",
        category_group_code: str = None,
        x: str = None,
        y: str = None,
        radius: int = None,
        rect: str = None,
        page: int = None,
        limit: int = None,
        sort: str = None,
    ) -> list[dict]:
        # https://developers.kakao.com/docs/latest/ko/local/dev-guide#search-by-keyword
        url = f"https://dapi.kakao.com/v2/local/search/keyword.{format}"
        params = dict(
            query=query,
            category_group_code=category_group_code,
            x=x,
            y=y,
            radius=radius,
            rect=rect,
            page=page,
            size=limit,
            sort=sort,
        )
        headers = dict(Authorization=f"KakaoAK {self.api_key}")
        resp = requests.get(url, params=params, headers=headers)
        parsed = resp.json()
        return parsed.get("documents", [])
